# Gravity
# Mar. 23, 2003
# Copyright (c) 2003 by David G. Drumright (ockham)

# Control the Y-tran so that it starts from a high
# point and bounces (with selectable damping)
# to a stop.

from Tkinter import *
root = Tk()

import poser
scene = poser.Scene()

#-----------------------------------------------------------------------------
# Global vars
#-----------------------------------------------------------------------------

global BounceProp
global Ball
global Cal
BounceProp = 0.0
Ball = 0
Cal = 0.0


#-----------------------------------------------------------------------------

def RunBounce():
    global BounceProp
    global Ball
    global Cal
    # From startframe to endframe, let the object fall at gravitational
    # acceleration.  When it hits Y=0, bounce upward to a proportion of
    # original height set by Damping, at a speed set by Damping but
    # countered by gravity; then bounce downward again until frames are done.

    StartFrame = 0
    EndFrame = scene.NumFrames() - 1  # later turn these to entries
    Yparm = Ball.Parameter("yTran")

    # Make a list of the original deltaY values, so we can add them
    # to the Disp at each point, both up and down.  (This will account
    # for original "loft" or "throwdown".)

    DeltaY = []
    for i in range(StartFrame+1, EndFrame):
        scene.SetFrame(i)
        NewY = Yparm.Value()
        scene.SetFrame(i-1)
        OldY = Yparm.Value()
        DeltaY.append(NewY-OldY)

    scene.SetFrame(StartFrame)
    StartingY = Yparm.Value()
    TempStartT = StartFrame
    # Now start going through frames, dropping the ball by the proper
    # amount for each frame. For convenience, figure SecsPerFrame once:
    SecsPerFrame = 1.0 / float(scene.FramesPerSecond())
    GoingDown = 1
    BounceHeight = StartingY
    for i in range(StartFrame,EndFrame):
        scene.SetFrame(i)
        Yval = Yparm.Value()
        T = float(i - TempStartT) * SecsPerFrame
        # Use Displacement = 1/2 * g * t^2.
        Disp = 0.5 * 9.8 * T * T * Cal

        if GoingDown:
            NewVal = BounceHeight - Disp + DeltaY[i-StartFrame]
            Yparm.SetValue(NewVal)
            if NewVal < 0.00001: # Switch directions.
               # Figure the upward tendency for next up, from the
               # starting height of this bounce / number of frames.
               # This will 'mirror' the net velocity of this fall.
               UpwardTendency = BounceHeight / (i - TempStartT + 1)
               GoingDown = 0
               # Now reset the time from which we measure Disp
               TempStartT = i
               BounceHeight = BounceHeight * BounceProp # decrease the bounce height for next up.
               if BounceHeight <= 0.00001: # If you set bounciness to 0, or if just about done, stop here.
                   break #Stop entirely
               else:
                   continue # Skip the lower else

        else: # upward.
            NewVal = Yval + (UpwardTendency * (i - TempStartT)) - Disp + DeltaY[i-StartFrame]
            Yparm.SetValue(NewVal)
            # Go up by mirrored amount, countered by gravity

            if NewVal < 0.00001: # Near bottom
               break
            elif NewVal >= BounceHeight: #Switch around
               GoingDown = 1
               # Reset the time from which we measure fall
               StartingT = i

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("Gravity bounce")

        self.PropLabel = Label(self.master,text="Name of prop to bounce:").grid(row=1, column=0)
        self.PropEntry = Entry(self.master)
        self.PropEntry.grid(row=1,column=1)

        self.CalLabel = Label(self.master,text="Start height in meters:").grid(row=2, column=0)
        self.CalEntry = Entry(self.master)
        self.CalEntry.grid(row=2, column=1)

        self.DampLabel = Label(self.master,text="Bounciness in percent:").grid(row=3, column=0)
        self.DampSlide = Scale(master, from_= 0.0, to=100.0,orient=HORIZONTAL,resolution=1.0)
        self.DampSlide.grid(row=3, column=1)
        self.DampSlide.set(0.0)

        # Go/nogo buttons.
        self.buttonGo = Button(self.master, text="Go", command=self.handleGo)
        self.buttonGo.grid(row=4, column=1)

        self.buttonCancel = Button(self.master, text="Cancel", command=self.handleCancel)
        self.buttonCancel.grid(row=4, column=2)

    # - - - - - - - - - - - - - - - - - -

    def handleGo(self):
        global Cal
        global Ball
        global BounceProp
        # Pick up the name, check it.
        try:
            TryName = self.PropEntry.get()
        except:
            raise "No name given"
        try:
            Ball = scene.Actor(TryName)
        except:
            raise "Given prop name is not present."
        # Figure Cal from actual starting position and
        # some Poser empirical values. If not given,
        # use standard.
        Yparm = Ball.Parameter("yTran")
        Yval = Yparm.Value()
        if Yval <= 0.0:
            raise "Prop is already at or below ground!"
        try:
            GivenCal = float(self.CalEntry.get())
        except: # No entry: set to use normal
            GivenCal = 0.0

        # Compare Wanted height with actual, to calibrate.
        # By observation, 0.44 Poser units = 1 actual meter, but we let
        # you set your own cal.  So, if GivenCal = 0.0, we use standard.
        # Cal should be Poser Units / Meter.
        if GivenCal == 0.0:
            Cal = 0.44 # Standard Poser units / meter
        else:  # Figure prop of actual height in Poser units to actual as given.
            Cal = Yval / GivenCal # Actual poser units / Given meters

        # Pick up bounciness from slider.
        B = self.DampSlide.get()
        BounceProp = B / 100.0
        RunBounce()
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def handleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

app = App(root, "")
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
